@@ -48,17 +48,16 @@
 #include "action-callbacks.h"
 #include "plugin.h"
 #include "search-box.h"
+#include "anjuta-bookmarks.h"
 
-#define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-document-manager.ui"
-#define PREFS_GLADE PACKAGE_DATA_DIR"/glade/anjuta-document-manager.glade"
+#define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-document-manager.xml"
+#define PREFS_BUILDER PACKAGE_DATA_DIR"/glade/anjuta-document-manager.ui"
 #define ICON_FILE "anjuta-document-manager-plugin-48.png"
 
 #define ANJUTA_PIXMAP_BOOKMARK_TOGGLE     "anjuta-bookmark-toggle"
-#define ANJUTA_PIXMAP_BOOKMARK_FIRST      "anjuta-bookmark-first"
-#define ANJUTA_PIXMAP_BOOKMARK_PREV       "anjuta-bookmark-prev"
-#define ANJUTA_PIXMAP_BOOKMARK_NEXT       "anjuta-bookmark-next"
-#define ANJUTA_PIXMAP_BOOKMARK_LAST       "anjuta-bookmark-last"
-#define ANJUTA_PIXMAP_BOOKMARK_CLEAR      "anjuta-bookmark-clear"
+#define ANJUTA_PIXMAP_BOOKMARK_PREV		  "anjuta-bookmark-prev"
+#define ANJUTA_PIXMAP_BOOKMARK_NEXT		  "anjuta-bookmark-next" 
+#define ANJUTA_PIXMAP_BOOKMARK_CLEAR	  "anjuta-bookmark-clear" 
 
 #define ANJUTA_PIXMAP_FOLD_TOGGLE         "anjuta-fold-toggle"
 #define ANJUTA_PIXMAP_FOLD_CLOSE          "anjuta-fold-close"
@@ -89,11 +88,9 @@
 #define ANJUTA_STOCK_BLOCK_START              "anjuta-block-start"
 #define ANJUTA_STOCK_BLOCK_END                "anjuta-block-end"
 #define ANJUTA_STOCK_BOOKMARK_TOGGLE          "anjuta-bookmark-toggle"
-#define ANJUTA_STOCK_BOOKMARK_FIRST           "anjuta-bookmark-first"
 #define ANJUTA_STOCK_BOOKMARK_PREV            "anjuta-bookmark-previous"
 #define ANJUTA_STOCK_BOOKMARK_NEXT            "anjuta-bookmark-next"
-#define ANJUTA_STOCK_BOOKMARK_LAST            "anjuta-bookmark-last"
-#define ANJUTA_STOCK_BOOKMARK_CLEAR           "anjuta-bookmark-clear"
+#define ANJUTA_STOCK_BOOKMARK_CLEAR           "anjuta-bookmark-clear" 
 #define ANJUTA_STOCK_GOTO_LINE				  "anjuta-goto-line"
 #define ANJUTA_STOCK_HISTORY_NEXT			  "anjuta-history-next"
 #define ANJUTA_STOCK_HISTORY_PREV			  "anjuta-history-prev"
@@ -149,10 +146,10 @@ static ShortcutMapping global_keymap[] = {
 static GtkActionEntry actions_file[] = {
   { "ActionFileSave", GTK_STOCK_SAVE, N_("_Save"), "<control>s",
 	N_("Save current file"), G_CALLBACK (on_save_activate)},
-  { "ActionFileSaveAs", GTK_STOCK_SAVE_AS, N_("Save _As..."),  "<shift><control>s",
+  { "ActionFileSaveAs", GTK_STOCK_SAVE_AS, N_("Save _As…"),  "<shift><control>s",
 	N_("Save the current file with a different name"),
     G_CALLBACK (on_save_as_activate)},
-  { "ActionFileSaveAll", NULL, N_("Save A_ll"), NULL,
+  { "ActionFileSaveAll", GTK_STOCK_SAVE, N_("Save A_ll"), "<shift><control>l",
 	N_("Save all currently open files, except new files"),
     G_CALLBACK (on_save_all_activate)},
   { "ActionFileClose", GTK_STOCK_CLOSE, N_("_Close File"), "<control>w",
@@ -168,7 +165,7 @@ static GtkActionEntry actions_file[] = {
 };
 
 static GtkActionEntry actions_print[] = {
-  { "ActionPrintFile", GTK_STOCK_PRINT, N_("_Print..."), "<control>p",
+  { "ActionPrintFile", GTK_STOCK_PRINT, N_("_Print…"), "<control>p",
 	N_("Print the current file"), G_CALLBACK (on_print_activate)},
   { "ActionPrintPreview",
 #ifdef GTK_STOCK_PRINT_PREVIEW
@@ -177,7 +174,7 @@ static GtkActionEntry actions_print[] = {
 	NULL
 #endif
 	, N_("_Print Preview"), NULL,
-	N_("Preview the current file in print-format"),
+	N_("Preview the current file in print format"),
     G_CALLBACK (on_print_preview_activate)},
 };
 
@@ -199,27 +196,19 @@ static GtkActionEntry actions_transform[] = {
 	N_("Convert End Of Line characters to Mac OS EOL (CR)"),
     G_CALLBACK (on_editor_command_eol_cr_activate)},
   { "ActionEditConvertEOL", NULL, N_("Convert EOL to Majority EOL"), NULL,
-	N_("Convert End Of Line characters to majority of the EOL found in the file"),
+	N_("Convert End Of Line characters to the most common EOL found in the file"),
     G_CALLBACK (on_transform_eolchars1_activate)},
 };
 
 static GtkActionEntry actions_select[] = {
-  { "ActionMenuEditSelect",  NULL, N_("_Select"), NULL, NULL, NULL}, /* menu title */
-  { "ActionEditSelectAll",
-#ifdef GTK_STOCK_SELECT_ALL
-	GTK_STOCK_SELECT_ALL
-#else
-	NULL
-#endif
-	, N_("Select _All"), "<control>a",
-	N_("Select all text in the editor"),
-    G_CALLBACK (on_editor_command_select_all_activate)},
-  { "ActionEditSelectToBrace", NULL, N_("Select to _Brace"), NULL,
-	N_("Select the text in the matching braces"),
-    G_CALLBACK (on_editor_command_select_to_brace_activate)},
-  { "ActionEditSelectBlock", ANJUTA_STOCK_BLOCK_SELECT, N_("Select _Code Block"),
-	 "<shift><control>b", N_("Select the current code block"),
-    G_CALLBACK (on_editor_command_select_block_activate)},
+	{ "ActionMenuEditSelect",  NULL, N_("_Select"), NULL, NULL, NULL}, /* menu title */
+	{ "ActionEditSelectAll",
+		GTK_STOCK_SELECT_ALL, N_("Select _All"), "<control>a",
+		N_("Select all text in the editor"),
+		G_CALLBACK (on_editor_command_select_all_activate)},
+	{ "ActionEditSelectBlock", ANJUTA_STOCK_BLOCK_SELECT, N_("Select _Code Block"),
+		"<shift><control>b", N_("Select the current code block"),
+		G_CALLBACK (on_editor_command_select_block_activate)}
 };
 
 static GtkActionEntry actions_comment[] = {
@@ -244,8 +233,8 @@ static GtkActionEntry actions_comment[] = {
 };
 
 static GtkActionEntry actions_navigation[] = {
-  { "ActionMenuGoto", NULL, N_("_Goto"), NULL, NULL, NULL},/* menu title */
-  { "ActionEditGotoLine", ANJUTA_STOCK_GOTO_LINE, N_("_Line Number..."),
+  { "ActionMenuGoto", NULL, N_("_Go to"), NULL, NULL, NULL},/* menu title */
+  { "ActionEditGotoLine", ANJUTA_STOCK_GOTO_LINE, N_("_Line Number…"),
 	"<control><alt>g", N_("Go to a particular line in the editor"),
     G_CALLBACK (on_goto_line_no1_activate)},
   { "ActionEditGotoMatchingBrace", GTK_STOCK_JUMP_TO, N_("Matching _Brace"),
@@ -258,10 +247,10 @@ static GtkActionEntry actions_navigation[] = {
 	"<control><alt>e", N_("Go to the end of the current block"),
     G_CALLBACK (on_goto_block_end1_activate)},
   { "ActionEditGotoHistoryPrev", ANJUTA_STOCK_HISTORY_PREV, N_("Previous _History"),
-	NULL, N_("Goto previous history"),
+	"<alt>Left", N_("Go to previous history"),
     G_CALLBACK (on_prev_history)},
   { "ActionEditGotoHistoryNext", ANJUTA_STOCK_HISTORY_NEXT, N_("Next Histor_y"),
-	 NULL, N_("Goto next history"),
+	"<alt>Right", N_("Go to next history"),
     G_CALLBACK (on_next_history)}
 };
 
@@ -293,7 +282,7 @@ static GtkActionEntry actions_edit[] = {
   { "ActionEditUndo", GTK_STOCK_UNDO, N_("U_ndo"), "<control>z",
 	N_("Undo the last action"),
     G_CALLBACK (on_editor_command_undo_activate)},
-  { "ActionEditRedo", GTK_STOCK_REDO, N_("_Redo"), "<control>r",
+  { "ActionEditRedo", GTK_STOCK_REDO, N_("_Redo"), "<shift><control>z",
 	N_("Redo the last undone action"),
     G_CALLBACK (on_editor_command_redo_activate)},
   { "ActionEditCut", GTK_STOCK_CUT, N_("C_ut"), "<control>x",
@@ -341,10 +330,10 @@ static GtkToggleActionEntry actions_view[] = {
 };
 
 static GtkActionEntry actions_zoom[] = {
-  { "ActionViewEditorZoomIn", GTK_STOCK_ZOOM_IN, N_("Zoom In"), "<control>plus",
+  { "ActionViewEditorZoomIn", GTK_STOCK_ZOOM_IN, N_("Zoom In"), "<control>KP_Add",
 	N_("Zoom in: Increase font size"),
     G_CALLBACK (on_zoom_in_text_activate)},
-  { "ActionViewEditorZoomOut", GTK_STOCK_ZOOM_OUT, N_("Zoom Out"), "<control>minus",
+  { "ActionViewEditorZoomOut", GTK_STOCK_ZOOM_OUT, N_("Zoom Out"), "<control>KP_Subtract",
 	N_("Zoom out: Decrease font size"),
     G_CALLBACK (on_zoom_out_text_activate)}
 };
@@ -365,28 +354,6 @@ static GtkActionEntry actions_format[] = {
     G_CALLBACK (on_editor_command_toggle_fold_activate)},
 };
 
-static GtkActionEntry actions_bookmark[] = {
-  { "ActionMenuBookmark", NULL, N_("Bookmar_k"), NULL, NULL, NULL},
-  { "ActionBookmarkToggle", ANJUTA_STOCK_BOOKMARK_TOGGLE, N_("_Toggle Bookmark"),
-	"<control>k", N_("Toggle a bookmark at the current line position"),
-    G_CALLBACK (on_editor_command_bookmark_toggle_activate)},
-  { "ActionBookmarkFirst", ANJUTA_STOCK_BOOKMARK_FIRST, N_("_First Bookmark"),
-	NULL, N_("Jump to the first bookmark in the file"),
-    G_CALLBACK (on_editor_command_bookmark_first_activate)},
-  { "ActionBookmarkPrevious", ANJUTA_STOCK_BOOKMARK_PREV, N_("_Previous Bookmark"),
-	"<control>comma", N_("Jump to the previous bookmark in the file"),
-    G_CALLBACK (on_editor_command_bookmark_prev_activate)},
-  { "ActionBookmarkNext", ANJUTA_STOCK_BOOKMARK_NEXT, N_("_Next Bookmark"),
-	"<control>period", N_("Jump to the next bookmark in the file"),
-    G_CALLBACK (on_editor_command_bookmark_next_activate)},
-  { "ActionBookmarkLast", ANJUTA_STOCK_BOOKMARK_LAST, N_("_Last Bookmark"),
-	NULL, N_("Jump to the last bookmark in the file"),
-    G_CALLBACK (on_editor_command_bookmark_last_activate)},
-  { "ActionBookmarkClear", ANJUTA_STOCK_BOOKMARK_CLEAR, N_("_Clear All Bookmarks"),
-	NULL, N_("Clear bookmarks"),
-    G_CALLBACK (on_editor_command_bookmark_clear_activate)},
-};
-
 static GtkActionEntry actions_documents[] = {
   {"ActionMenuDocuments", NULL, N_("_Documents"), NULL, NULL, NULL},
   { "ActionDocumentsPrevious", GTK_STOCK_GO_BACK, N_("Previous Document"),
@@ -397,6 +364,22 @@ static GtkActionEntry actions_documents[] = {
     G_CALLBACK (on_next_document)},
 };
 
+static GtkActionEntry actions_bookmarks[] = {
+	{ "ActionMenuBookmark", NULL, N_("Bookmar_k"), NULL, NULL, NULL},
+ 	{ "ActionBookmarkAdd", GTK_STOCK_ADD, N_("_Add Bookmark"),
+		 "<control>k", N_("Add a bookmark at the current line position"),
+	G_CALLBACK (on_bookmark_add_activate)},
+	{ "ActionBookmarkPrev", ANJUTA_STOCK_BOOKMARK_PREV, N_("_Previous Bookmark"),
+	"<control>comma", N_("Jump to the previous bookmark in the file"),
+	G_CALLBACK (on_bookmark_prev_activate)},
+ 	{ "ActionBookmarkNext", ANJUTA_STOCK_BOOKMARK_NEXT, N_("_Next Bookmark"),
+ 	"<control>period", N_("Jump to the next bookmark in the file"),
+ 	G_CALLBACK (on_bookmark_next_activate)},
+ 	{ "ActionBookmarksClear", ANJUTA_STOCK_BOOKMARK_CLEAR, N_("_Clear All Bookmarks"),
+ 	NULL, N_("Clear bookmarks"),
+ 	G_CALLBACK (on_bookmarks_clear_activate)},
+ }; 
+
 struct ActionGroupInfo {
 	GtkActionEntry *group;
 	gint size;
@@ -423,9 +406,9 @@ static struct ActionGroupInfo action_groups[] = {
 	{ actions_zoom, G_N_ELEMENTS (actions_zoom), "ActionGroupEditorZoom", N_("Editor zoom operations") },
 	{ actions_style, G_N_ELEMENTS (actions_style), "ActionGroupEditorStyle", N_("Editor syntax highlighting styles") },
 	{ actions_format, G_N_ELEMENTS (actions_format), "ActionGroupEditorFormat", N_("Editor text formating") },
-	{ actions_bookmark, G_N_ELEMENTS (actions_bookmark), "ActionGroupEditorBookmark", N_("Editor bookmarks") },
 	{ actions_search, G_N_ELEMENTS (actions_search), "ActionGroupEditorSearch", N_("Simple searching") },
-	{ actions_documents, G_N_ELEMENTS (actions_documents), "ActionGroupDocuments", N_("Documents") }
+	{ actions_documents, G_N_ELEMENTS (actions_documents), "ActionGroupDocuments", N_("Documents") },
+	{ actions_bookmarks, G_N_ELEMENTS (actions_bookmarks), "ActionGroupBookmarks", N_("Bookmarks") }
 };
 
 static struct ActionToggleGroupInfo action_toggle_groups[] = {
@@ -509,7 +492,7 @@ update_title (DocmanPlugin* doc_plugin)
 			if (dir)
 				title = g_strdup_printf ("%s (%s)", real_filename, dir);
 			else
-				title = g_strdup_printf ("%s (%s)", real_filename, dir);				
+				title = g_strdup_printf ("%s", real_filename);
 		}
 		g_free (real_filename);
 		g_free (dir);
@@ -533,7 +516,7 @@ value_added_project_root_uri (AnjutaPlugin *plugin, const gchar *name,
 
 	doc_plugin = ANJUTA_PLUGIN_DOCMAN (plugin);
 	
-	DEBUG_PRINT ("Project added");
+	DEBUG_PRINT ("%s", "Project added");
 	
 	
 	g_free (doc_plugin->project_name);
@@ -565,7 +548,7 @@ value_removed_project_root_uri (AnjutaPlugin *plugin, const gchar *name,
 
 	doc_plugin = ANJUTA_PLUGIN_DOCMAN (plugin);
 	
-	DEBUG_PRINT ("Project removed");
+	DEBUG_PRINT ("%s", "Project removed");
 	
 	g_free (doc_plugin->project_name);
 	doc_plugin->project_name = NULL;
@@ -594,7 +577,7 @@ ui_states_init (AnjutaPlugin *plugin)
 		GtkAction *action;
 		gboolean state;
 		
-		state = anjuta_preferences_get_int (eplugin->prefs, prefs[i]);
+		state = anjuta_preferences_get_bool (eplugin->prefs, prefs[i]);
 		action = anjuta_ui_get_action (eplugin->ui, "ActionGroupEditorView",
 									   actions_view[i].name);
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), state);
@@ -620,7 +603,7 @@ ui_give_shorter_names (AnjutaPlugin *plugin)
 	g_object_set (G_OBJECT (action), "is-important", TRUE, NULL);
 	action = anjuta_ui_get_action (ui, "ActionGroupEditorNavigate",
 								   "ActionEditGotoLine");
-	g_object_set (G_OBJECT (action), "short-label", _("Goto"), NULL);
+	g_object_set (G_OBJECT (action), "short-label", _("Go to"), NULL);
 }
 
 static void
@@ -743,10 +726,6 @@ update_document_ui_interface_items (AnjutaPlugin *plugin, IAnjutaDocument *doc)
 								   "ActionEditSelectAll");
 	g_object_set (G_OBJECT (action), "visible", flag, "sensitive", flag, NULL);
 	
-	action = anjuta_ui_get_action (ui, "ActionGroupEditorSelect",
-								   "ActionEditSelectToBrace");
-	g_object_set (G_OBJECT (action), "visible", flag, "sensitive", flag, NULL);
-	
 	action = anjuta_ui_get_action (ui, "ActionGroupEditorSelect",
 								   "ActionEditSelectBlock");
 	g_object_set (G_OBJECT (action), "visible", flag, "sensitive", flag, NULL);
@@ -869,7 +848,7 @@ update_document_ui (AnjutaPlugin *plugin, IAnjutaDocument *doc)
 }
 
 static void
-on_document_update_save_ui (IAnjutaDocument *doc, gboolean entered,
+on_document_update_save_ui (IAnjutaDocument *doc,
 							AnjutaPlugin *plugin)
 {
 	update_document_ui_save_items (plugin, doc);
@@ -894,11 +873,9 @@ register_stock_icons (AnjutaPlugin *plugin)
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_INDENT_INC, ANJUTA_STOCK_INDENT_INC);
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BLOCK_SELECT, ANJUTA_STOCK_BLOCK_SELECT);
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BOOKMARK_TOGGLE, ANJUTA_STOCK_BOOKMARK_TOGGLE);
-	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BOOKMARK_FIRST, ANJUTA_STOCK_BOOKMARK_FIRST);
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BOOKMARK_PREV, ANJUTA_STOCK_BOOKMARK_PREV);
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BOOKMARK_NEXT, ANJUTA_STOCK_BOOKMARK_NEXT);
-	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BOOKMARK_LAST, ANJUTA_STOCK_BOOKMARK_LAST);
-	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BOOKMARK_CLEAR, ANJUTA_STOCK_BOOKMARK_CLEAR);
+	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BOOKMARK_CLEAR, ANJUTA_STOCK_BOOKMARK_CLEAR); 
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BLOCK_START, ANJUTA_STOCK_BLOCK_START);
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BLOCK_END, ANJUTA_STOCK_BLOCK_END);
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_GOTO_LINE, ANJUTA_STOCK_GOTO_LINE);
@@ -923,25 +900,6 @@ update_status (DocmanPlugin *plugin, IAnjutaEditor *te)
 		if (status == NULL)
 			return;
 		
-		/* TODO: Implement this in IAnjutaEditor some kind
-		gint editor_mode;
-		editor_mode =  scintilla_send_message (SCINTILLA (te->widgets.editor),
-											   SCI_GETEOLMODE, 0, 0);
-		switch (editor_mode) {
-			case SC_EOL_CRLF:
-				mode = g_strdup(_("DOS (CRLF)"));
-				break;
-			case SC_EOL_LF:
-				mode = g_strdup(_("Unix (LF)"));
-				break;
-			case SC_EOL_CR:
-				mode = g_strdup(_("Mac (CR)"));
-				break;
-			default:
-				mode = g_strdup(_("Unknown"));
-				break;
-		}*/
-		
 		zoom = anjuta_preferences_get_int (plugin->prefs, TEXT_ZOOM_FACTOR);
 		line = ianjuta_editor_get_lineno (te, NULL);
 		col = ianjuta_editor_get_column (te, NULL);
@@ -954,7 +912,12 @@ update_status (DocmanPlugin *plugin, IAnjutaEditor *te)
 		{
 			edit = g_strdup (_("INS"));
 		}
-		anjuta_status_set_default (status, _("Zoom"), "%d", zoom);
+		
+		if (IANJUTA_IS_EDITOR_ZOOM(te))
+			anjuta_status_set_default (status, _("Zoom"), "%d", zoom);
+		else
+			anjuta_status_set_default (status, _("Zoom"), NULL);
+		
 		anjuta_status_set_default (status, _("Line"), "%04d", line);
 		anjuta_status_set_default (status, _("Col"), "%03d", col);
 		anjuta_status_set_default (status, _("Mode"), edit);
@@ -968,11 +931,7 @@ update_status (DocmanPlugin *plugin, IAnjutaEditor *te)
 		status = anjuta_shell_get_status (ANJUTA_PLUGIN (plugin)->shell, NULL);
 		if (status)
 		{
-			anjuta_status_set_default (status, _("Zoom"), NULL);
-			anjuta_status_set_default (status, _("Line"), NULL);
-			anjuta_status_set_default (status, _("Col"), NULL);
-			anjuta_status_set_default (status, _("Mode"), NULL);
-			/* anjuta_status_set_default (status, _("EOL"), NULL); */
+			anjuta_status_set (status, "");
 		}
 	}
 }
@@ -1030,8 +989,7 @@ create_highlight_submenu (DocmanPlugin *plugin, IAnjutaEditor *editor)
 	const GList *languages, *node;
 	GList *sorted_languages;
 	GtkWidget *submenu;
-	GtkWidget *menuitem;
-	
+	GtkWidget *auto_menuitem;	
 	submenu = gtk_menu_new ();
 	
 	if (!editor || !IANJUTA_IS_EDITOR_LANGUAGE (editor))
@@ -1042,12 +1000,13 @@ create_highlight_submenu (DocmanPlugin *plugin, IAnjutaEditor *editor)
 		return NULL;
 	
 	/* Automatic highlight menu */
-	menuitem = gtk_menu_item_new_with_mnemonic (_("Automatic"));
-	g_signal_connect (G_OBJECT (menuitem), "activate",
+	auto_menuitem = gtk_radio_menu_item_new_with_mnemonic (NULL, _("Automatic"));
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(auto_menuitem), TRUE);
+	g_signal_connect (G_OBJECT (auto_menuitem), "activate",
 					  G_CALLBACK (on_force_hilite_activate),
 					  plugin);
-	g_object_set_data(G_OBJECT(menuitem), "language_code", "auto-detect");
-	gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menuitem);
+	g_object_set_data(G_OBJECT(auto_menuitem), "language_code", "auto-detect");
+	gtk_menu_shell_append (GTK_MENU_SHELL (submenu), auto_menuitem);
 	gtk_menu_shell_append (GTK_MENU_SHELL (submenu), gtk_separator_menu_item_new());
 
 	/* Sort languages so they appear alphabetically in the menu. */
@@ -1064,7 +1023,8 @@ create_highlight_submenu (DocmanPlugin *plugin, IAnjutaEditor *editor)
 		/* Should fix #493583 */
 		if (name != NULL)
 		{
-			menuitem = gtk_menu_item_new_with_mnemonic (name);
+			GtkWidget *menuitem;
+			menuitem = gtk_radio_menu_item_new_with_mnemonic_from_widget (GTK_RADIO_MENU_ITEM(auto_menuitem), name);
 			g_object_set_data_full (G_OBJECT (menuitem), "language_code",
 									g_strdup (lang),
 									(GDestroyNotify)g_free);
@@ -1072,6 +1032,7 @@ create_highlight_submenu (DocmanPlugin *plugin, IAnjutaEditor *editor)
 							  G_CALLBACK (on_force_hilite_activate),
 							  plugin);
 			gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menuitem);
+			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menuitem), FALSE);
 		}
 		node = g_list_next (node);
 	}
@@ -1092,7 +1053,7 @@ on_document_added (AnjutaDocman *docman, IAnjutaDocument *doc,
 	g_signal_connect (G_OBJECT (doc), "update_ui",
 					  G_CALLBACK (on_document_update_ui),
 					  docman_plugin);
-	g_signal_connect (G_OBJECT (doc), "save_point",
+	g_signal_connect (G_OBJECT (doc), "update-save-ui",
 					  G_CALLBACK (on_document_update_save_ui),
 					  plugin);
 	/* Present the vbox as this is the widget that was added to the shell */
@@ -1159,7 +1120,7 @@ unload_unused_support_plugins (DocmanPlugin* docman_plugin,
 		AnjutaPlugin* plugin = ANJUTA_PLUGIN (node->data);
 		if (!g_list_find (needed_plugins, plugin))
 		{
-			DEBUG_PRINT ("Unloading plugin");
+			DEBUG_PRINT ("%s", "Unloading plugin");
 			anjuta_plugin_deactivate (plugin);
 		}
 	}
@@ -1170,6 +1131,7 @@ static void
 on_document_changed (AnjutaDocman *docman, IAnjutaDocument *doc,
 					 AnjutaPlugin *plugin)
 {
+	DocmanPlugin *docman_plugin;
 	update_document_ui (plugin, doc);
 	
 	if (doc)
@@ -1187,17 +1149,15 @@ on_document_changed (AnjutaDocman *docman, IAnjutaDocument *doc,
 		anjuta_shell_remove_value (plugin->shell, IANJUTA_DOCUMENT_MANAGER_CURRENT_DOCUMENT,
 								   NULL);
 	}
-
+	docman_plugin = ANJUTA_PLUGIN_DOCMAN (plugin);
+	
 	if (doc && IANJUTA_IS_EDITOR (doc))
 	{
-		DocmanPlugin *docman_plugin;
-
-		docman_plugin = ANJUTA_PLUGIN_DOCMAN (plugin);
 		update_status (docman_plugin, IANJUTA_EDITOR (doc));
 
 		if (IANJUTA_IS_EDITOR_LANGUAGE (doc))
 		{
-			DEBUG_PRINT("Beginning language support");
+			DEBUG_PRINT("%s", "Beginning language support");
 			AnjutaPluginManager *plugin_manager;
 			IAnjutaLanguage *lang_manager;
 			GList *new_support_plugins, *support_plugin_descs, *needed_plugins, *node;
@@ -1216,7 +1176,7 @@ on_document_changed (AnjutaDocman *docman, IAnjutaDocument *doc,
 															  NULL);
 			if (!language)
 			{
-				DEBUG_PRINT ("Unloading all language support plugins");
+				DEBUG_PRINT ("%s", "Unloading all language support plugins");
 				/* Unload all language support plugins */
 				/* Copy the list because the "deactivate"-signal handler modifies
 				 * the original list */
@@ -1268,6 +1228,8 @@ on_document_changed (AnjutaDocman *docman, IAnjutaDocument *doc,
 			}
 		}
 	}
+	else
+		update_status (docman_plugin, NULL);
 out:
 	update_title (ANJUTA_PLUGIN_DOCMAN (plugin));
 }
@@ -1316,7 +1278,7 @@ on_window_key_press_event (AnjutaShell *shell,
 		else
 			cur_page = cur_page ? cur_page - 1 : pages_nb -1;
 
-		gtk_notebook_set_page (notebook, cur_page);
+		gtk_notebook_set_current_page (notebook, cur_page);
 
 		break;
 	}
@@ -1329,7 +1291,7 @@ on_window_key_press_event (AnjutaShell *shell,
 
 			if (!notebook->children)
 				return FALSE;
-			gtk_notebook_set_page(notebook, page_req);
+			gtk_notebook_set_current_page(notebook, page_req);
 		}
 		else
 			return FALSE;
@@ -1364,7 +1326,7 @@ on_window_key_release_event (AnjutaShell *shell,
 		gint cur_page;
 		plugin->g_tabbing = FALSE;
 		
-		if (anjuta_preferences_get_int (plugin->prefs,
+		if (anjuta_preferences_get_bool (plugin->prefs,
 										EDITOR_TABS_RECENT_FIRST))
 		{
 			/*
@@ -1379,6 +1341,17 @@ on_window_key_release_event (AnjutaShell *shell,
 	return FALSE;
 }
 
+static void
+on_session_load (AnjutaShell *shell, AnjutaSessionPhase phase,
+				 AnjutaSession *session, DocmanPlugin *plugin)
+{
+	if (phase != ANJUTA_SESSION_PHASE_NORMAL)
+		return;
+	
+	anjuta_bookmarks_session_load (ANJUTA_BOOKMARKS (plugin->bookmarks),
+								   session);
+}
+
 static void
 on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase,
 				 AnjutaSession *session, DocmanPlugin *plugin)
@@ -1423,6 +1396,9 @@ on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase,
 		g_list_foreach (files, (GFunc)g_free, NULL);
 		g_list_free (files);
 	}
+	
+	anjuta_bookmarks_session_save (ANJUTA_BOOKMARKS (plugin->bookmarks),
+								   session);
 }
 
 static gboolean
@@ -1474,8 +1450,8 @@ on_save_prompt (AnjutaShell *shell, AnjutaSavePrompt *save_prompt,
 static void
 docman_plugin_set_tab_pos (DocmanPlugin *ep)
 {
-	if (anjuta_preferences_get_int_with_default (ep->prefs, EDITOR_TABS_HIDE,
-												 1))
+	if (anjuta_preferences_get_bool_with_default (ep->prefs, EDITOR_TABS_HIDE,
+												 TRUE))
 	{
 		gtk_notebook_set_show_tabs (GTK_NOTEBOOK (ep->docman), FALSE);
 	}
@@ -1505,8 +1481,8 @@ docman_plugin_set_tab_pos (DocmanPlugin *ep)
 }
 
 static void
-on_gconf_notify_prefs (GConfClient *gclient, guint cnxn_id,
-					   GConfEntry *entry, gpointer user_data)
+on_notify_prefs (AnjutaPreferences* prefs, 
+                 const gchar* key, gint value, gpointer user_data)
 {
 	DocmanPlugin *ep = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman_plugin_set_tab_pos (ep);
@@ -1529,7 +1505,7 @@ on_docman_auto_save (gpointer data)
 		return FALSE;
 
 	prefs = anjuta_shell_get_preferences (docman->shell, NULL);
-	if (anjuta_preferences_get_int (prefs, SAVE_AUTOMATIC) == FALSE)
+	if (anjuta_preferences_get_bool (prefs, SAVE_AUTOMATIC) == FALSE)
 	{
 		plugin->autosave_on = FALSE;
 		return FALSE;
@@ -1544,7 +1520,8 @@ on_docman_auto_save (gpointer data)
 		for (node = buffers; node != NULL; node = g_list_next (node))
 		{
 			doc = IANJUTA_DOCUMENT (node->data);
-			if (ianjuta_file_savable_is_dirty (IANJUTA_FILE_SAVABLE (doc), NULL))
+			if (ianjuta_file_savable_is_dirty (IANJUTA_FILE_SAVABLE (doc), NULL) &&
+				!ianjuta_file_savable_is_conflict (IANJUTA_FILE_SAVABLE (doc), NULL))
 			{
 				GFile* file = ianjuta_file_get_file (IANJUTA_FILE (doc), NULL);
 				if (file)
@@ -1577,22 +1554,21 @@ on_docman_auto_save (gpointer data)
 }
 
 static void
-on_gconf_notify_timer (GConfClient *gclient, guint cnxn_id,
-					   GConfEntry *entry, gpointer user_data)
+on_notify_timer (AnjutaPreferences* prefs,
+                 const gchar* key,
+                 gboolean value,
+                 gpointer user_data)
 {
 	DocmanPlugin *plugin;
 	AnjutaDocman *docman;
-	AnjutaPreferences* prefs;
 	gint auto_save_timer;
 	gboolean auto_save;
 	
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-
-	prefs = anjuta_shell_get_preferences (docman->shell, NULL);
 	
 	auto_save_timer = anjuta_preferences_get_int(prefs, AUTOSAVE_TIMER);
-	auto_save = anjuta_preferences_get_int(prefs, SAVE_AUTOMATIC);
+	auto_save = anjuta_preferences_get_bool(prefs, SAVE_AUTOMATIC);
 	
 	if (auto_save)
 	{
@@ -1602,23 +1578,15 @@ on_gconf_notify_timer (GConfClient *gclient, guint cnxn_id,
 			{
 				g_source_remove (plugin->autosave_id);
 				plugin->autosave_id =
-#if GLIB_CHECK_VERSION (2,14,0)
 					g_timeout_add_seconds (auto_save_timer * 60,
-#else
-					g_timeout_add (auto_save_timer * 60000,
-#endif
-									on_docman_auto_save, plugin);
+										   on_docman_auto_save, plugin);
 			}
 		}
 		else
 		{
 			plugin->autosave_id =
-#if GLIB_CHECK_VERSION (2,14,0)
 				g_timeout_add_seconds (auto_save_timer * 60,
-#else
-				g_timeout_add (auto_save_timer * 60000,
-#endif
-							 on_docman_auto_save, plugin);
+									   on_docman_auto_save, plugin);
 		}
 		plugin->autosave_it = auto_save_timer;
 		plugin->autosave_on = TRUE;
@@ -1630,8 +1598,8 @@ on_gconf_notify_timer (GConfClient *gclient, guint cnxn_id,
 	}
 }
 
-#define REGISTER_NOTIFY(key, func) \
-	notify_id = anjuta_preferences_notify_add (ep->prefs, \
+#define REGISTER_NOTIFY(key, func, type) \
+	notify_id = anjuta_preferences_notify_add_##type (ep->prefs, \
 											   key, func, ep, NULL); \
 	ep->gconf_notify_ids = g_list_prepend (ep->gconf_notify_ids, \
 										   GUINT_TO_POINTER (notify_id));
@@ -1640,12 +1608,12 @@ prefs_init (DocmanPlugin *ep)
 {
 	guint notify_id;
 	docman_plugin_set_tab_pos (ep);
-	REGISTER_NOTIFY (EDITOR_TABS_HIDE, on_gconf_notify_prefs);
-	REGISTER_NOTIFY (EDITOR_TABS_POS, on_gconf_notify_prefs);
-	REGISTER_NOTIFY (AUTOSAVE_TIMER, on_gconf_notify_timer);
-	REGISTER_NOTIFY (SAVE_AUTOMATIC, on_gconf_notify_timer);
+	REGISTER_NOTIFY (EDITOR_TABS_HIDE, on_notify_prefs, bool);
+	REGISTER_NOTIFY (EDITOR_TABS_POS, on_notify_prefs, int);
+	REGISTER_NOTIFY (AUTOSAVE_TIMER, on_notify_timer, bool);
+	REGISTER_NOTIFY (SAVE_AUTOMATIC, on_notify_timer, bool);
 	
-	on_gconf_notify_timer(NULL,0,NULL, ep);
+	on_notify_timer(anjuta_preferences_default(), NULL, FALSE, ep);
 }
 
 static void
@@ -1674,7 +1642,7 @@ activate_plugin (AnjutaPlugin *plugin)
 	static gboolean initialized = FALSE;
 	GList *actions, *act;
 	
-	DEBUG_PRINT ("DocmanPlugin: Activating document manager plugin...");
+	DEBUG_PRINT ("%s", "DocmanPlugin: Activating document manager plugin…");
 	
 	dplugin = ANJUTA_PLUGIN_DOCMAN (plugin);
 	dplugin->ui = anjuta_shell_get_ui (plugin->shell, NULL);
@@ -1740,7 +1708,7 @@ activate_plugin (AnjutaPlugin *plugin)
 	dplugin->uiid = anjuta_ui_merge (ui, UI_FILE);
 	dplugin->vbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (dplugin->vbox);
-	gtk_box_pack_start_defaults (GTK_BOX(dplugin->vbox), docman);
+	gtk_box_pack_start (GTK_BOX(dplugin->vbox), docman, TRUE, TRUE, 0);
 	anjuta_shell_add_widget_full (plugin->shell, dplugin->vbox,
 							 "AnjutaDocumentManager", _("Documents"),
 							 "editor-plugin-icon", 
@@ -1768,10 +1736,17 @@ activate_plugin (AnjutaPlugin *plugin)
 	/* Connect to save session */
 	g_signal_connect (G_OBJECT (plugin->shell), "save-session",
 					  G_CALLBACK (on_session_save), plugin);
+	/* Connect to load session */
+	g_signal_connect (G_OBJECT (plugin->shell), "load-session",
+					  G_CALLBACK (on_session_load), plugin);
+	
 	/* Connect to save prompt */
 	g_signal_connect (G_OBJECT (plugin->shell), "save-prompt",
 					  G_CALLBACK (on_save_prompt), plugin);
 	
+	/* Add bookmarks widget */
+	dplugin->bookmarks = G_OBJECT(anjuta_bookmarks_new (dplugin));
+	
 	dplugin->project_watch_id =
 		anjuta_plugin_add_watch (plugin, IANJUTA_PROJECT_MANAGER_PROJECT_ROOT_URI,
 								 value_added_project_root_uri,
@@ -1793,7 +1768,7 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	AnjutaStatus *status;
 	GList *node;
 	
-	DEBUG_PRINT ("DocmanPlugin: Deactivating document manager plugin...");
+	DEBUG_PRINT ("%s", "DocmanPlugin: Deactivating document manager plugin…");
 	
 	eplugin = ANJUTA_PLUGIN_DOCMAN (plugin);
 
@@ -1821,6 +1796,7 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	/* Widget is removed from the container when destroyed */
 	gtk_widget_destroy (eplugin->docman);
 	gtk_widget_destroy (eplugin->search_box);	/* the default searchbox instance may still be unparented */
+	g_object_unref (eplugin->bookmarks);
 	anjuta_ui_unmerge (ui, eplugin->uiid);
 	node = eplugin->action_groups;
 	while (node)
@@ -1915,17 +1891,6 @@ ianjuta_docman_set_current_document (IAnjutaDocumentManager *plugin,
 	anjuta_docman_set_current_document (docman, doc);
 }
 
-static void
-ianjuta_docman_set_message_area (IAnjutaDocumentManager *plugin,
-								 IAnjutaDocument *doc,
-								 GtkWidget *message_area,
-								 GError **e)
-{
-	AnjutaDocman *docman;
-	docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
-	anjuta_docman_set_message_area (docman, doc, message_area);
-}
-
 static GList*
 ianjuta_docman_get_doc_widgets (IAnjutaDocumentManager *plugin, GError **e)
 {
@@ -2015,6 +1980,16 @@ ianjuta_docman_remove_document(IAnjutaDocumentManager *plugin,
 	return ret_val;
 }
 
+static void
+ianjuta_docman_add_bookmark (IAnjutaDocumentManager* plugin,
+									  GFile* file,
+									  gint  line,
+									  GError **e)
+{
+	AnjutaBookmarks* bookmarks = ANJUTA_BOOKMARKS (ANJUTA_PLUGIN_DOCMAN(plugin)->bookmarks);
+	anjuta_bookmarks_add_file (bookmarks, file, line, NULL);
+}
+
 static void
 ianjuta_document_manager_iface_init (IAnjutaDocumentManagerIface *iface)
 {
@@ -2028,7 +2003,7 @@ ianjuta_document_manager_iface_init (IAnjutaDocumentManagerIface *iface)
 	iface->goto_file_line_mark = ianjuta_docman_goto_file_line_mark;
 	iface->remove_document = ianjuta_docman_remove_document;
 	iface->set_current_document = ianjuta_docman_set_current_document;
-	iface->set_message_area = ianjuta_docman_set_message_area;
+	iface->add_bookmark = ianjuta_docman_add_bookmark;
 }
 
 /* Implement IAnjutaFile interface */
@@ -2092,7 +2067,7 @@ isaveable_save (IAnjutaFileSavable* plugin, GError** e)
 static void
 isavable_save_as (IAnjutaFileSavable* plugin, GFile* file, GError** e)
 {
-	DEBUG_PRINT("save_as: Not implemented in DocmanPlugin");
+	DEBUG_PRINT("%s", "save_as: Not implemented in DocmanPlugin");
 }
 
 static gboolean
@@ -2126,7 +2101,7 @@ isavable_is_dirty (IAnjutaFileSavable* plugin, GError **e)
 static void
 isavable_set_dirty(IAnjutaFileSavable* plugin, gboolean dirty, GError** e)
 {
-	DEBUG_PRINT("set_dirty: Not implemented in DocmanPlugin");
+	DEBUG_PRINT("%s", "set_dirty: Not implemented in DocmanPlugin");
 }
 
 static void
@@ -2141,15 +2116,20 @@ isavable_iface_init (IAnjutaFileSavableIface *iface)
 static void
 ipreferences_merge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
 {
-	GladeXML* gxml;
+	GError* error = NULL;
+	GtkBuilder* bxml = gtk_builder_new ();
 	
 	/* Add preferences */
-	gxml = glade_xml_new (PREFS_GLADE, "preferences_dialog", NULL);
-		
-	anjuta_preferences_add_page (prefs,
-									gxml, "Documents", _("Documents"),  ICON_FILE);
-				
-	g_object_unref (G_OBJECT (gxml));
+	if (!gtk_builder_add_from_file (bxml, PREFS_BUILDER, &error))
+	{
+		g_warning ("Couldn't load builder file: %s", error->message);
+		g_error_free (error);
+	}
+
+	anjuta_preferences_add_from_builder (prefs,
+									bxml, "Documents", _("Documents"),  ICON_FILE);
+
+	g_object_unref (G_OBJECT (bxml));
 }
 
 static void
